==> syd-3.15.2.tar.gz: syd-v3.15.2/src/syd-sha.rs <==
//
// syd: seccomp and landlock based application sandbox with support for namespaces
// src/syd-sha.rs: Calculate MD5 or SHA1,3-{256,384,512} checksum of the given file or standard input.
//
// Copyright (c) 2024 Ali Polatel <alip@chesswob.org>
//
// SPDX-License-Identifier: GPL-3.0-or-later

use std::{
    env,
    fs::File,
    io::{BufReader, Write},
    process::ExitCode,
};

use getargs::{Opt, Options};
use syd::hash::HashFunc;

fn main() -> ExitCode {
    #[allow(clippy::disallowed_methods)]
    syd::set_sigpipe_dfl().expect("SIGPIPE");

    let args = env::args().skip(1).collect::<Vec<_>>();
    let mut opts = Options::new(args.iter().map(String::as_str));
    let mut func = HashFunc::Sha512;
    let mut bino = false; // Binary output?

    // SAFETY: We panic on parse errors.
    #[allow(clippy::disallowed_methods)]
    while let Some(opt) = opts.next_opt().expect("next opt") {
        match opt {
            Opt::Short('h') => {
                help();
                return ExitCode::SUCCESS;
            }
            Opt::Short('b') => {
                bino = true;
            }
            Opt::Short('x') => {
                bino = false;
            }
            Opt::Short('1') => {
                func = HashFunc::Sha1;
            }
            Opt::Short('2') => {
                func = HashFunc::Sha256;
            }
            Opt::Short('3') => {
                func = HashFunc::Sha384;
            }
            Opt::Short('5') => {
                func = HashFunc::Sha512;
            }
            Opt::Short('m') => {
                func = HashFunc::Md5;
            }
            _ => {
                eprintln!("Unknown option: {opt:?}!");
                return ExitCode::FAILURE;
            }
        }
    }

    #[allow(clippy::disallowed_methods)]
    match opts.positionals().next() {
        None | Some("-") => {
            if bino {
                // Binary output for standard input
                std::io::stdout()
                    .write_all(&syd::hash::hash(std::io::stdin().lock(), func).expect("hash"))
                    .expect("write to stdout");
            } else {
                // Hexadecimal output for standard input
                println!(
                    "{} -",
                    const_hex::encode(
                        syd::hash::hash(std::io::stdin().lock(), func).expect("hash")
                    )
                );
            }
        }
        Some(path) => {
            let file = BufReader::new(File::open(path).expect("open file"));
            if bino {
                // Binary output for file input
                std::io::stdout()
                    .write_all(&syd::hash::hash(file, func).expect("hash"))
                    .expect("write to stdout");
            } else {
                // Hexadecimal output for file input
                println!(
                    "{} {path}",
                    const_hex::encode(syd::hash::hash(file, func).expect("hash"))
                );
            }
        }
    }

    ExitCode::SUCCESS
}

fn help() {
    println!("Usage: syd-sha [-h1235m] <file|->");
    println!("Given a file, print the SHA3-512 checksum of the file.");
    println!("Given no positional arguments, calculate the SHA3-512 checksum of standard input.");
    println!("Use -m to calculate MD5 instead of SHA3-512 (\x1b[91minsecure\x1b[0m, \x1b[96mPortage\x1b[0m/\x1b[95mPaludis\x1b[0m vdb compat).");
    println!("Use -1 to calculate SHA1 instead of SHA3-512 (\x1b[91minsecure\x1b[0m).");
    println!("Use -2 to calculate SHA3-256 instead of SHA3-512.");
    println!("Use -3 to calculate SHA3-384 instead of SHA3-512.");
    println!("Use -b to print binary output rather than hex-encoded string.");
}

==> syd-3.15.5.tar.gz: syd-v3.15.5/src/syd-sha.rs <==
//
// syd: seccomp and landlock based application sandbox with support for namespaces
// src/syd-sha.rs: Calculate MD5 or SHA1,3-{256,384,512} checksum of the given file or standard input.
//
// Copyright (c) 2024 Ali Polatel <alip@chesswob.org>
//
// SPDX-License-Identifier: GPL-3.0-or-later

use std::{
    env,
    fs::File,
    io::{BufReader, Write},
    process::ExitCode,
};

use getargs::{Opt, Options};
use syd::hash::HashFunc;

fn main() -> ExitCode {
    #[allow(clippy::disallowed_methods)]
    syd::set_sigpipe_dfl().expect("SIGPIPE");

    let args = env::args().skip(1).collect::<Vec<_>>();
    let mut opts = Options::new(args.iter().map(String::as_str));
    let mut func = HashFunc::Sha512;
    let mut bino = false; // Binary output?

    // SAFETY: We panic on parse errors.
    #[allow(clippy::disallowed_methods)]
    while let Some(opt) = opts.next_opt().expect("next opt") {
        match opt {
            Opt::Short('h') => {
                help();
                return ExitCode::SUCCESS;
            }
            Opt::Short('b') => {
                bino = true;
            }
            Opt::Short('x') => {
                bino = false;
            }
            Opt::Short('1') => {
                func = HashFunc::Sha1;
            }
            Opt::Short('2') => {
                func = HashFunc::Sha256;
            }
            Opt::Short('3') => {
                func = HashFunc::Sha384;
            }
            Opt::Short('5') => {
                func = HashFunc::Sha512;
            }
            Opt::Short('m') => {
                func = HashFunc::Md5;
            }
            _ => {
                eprintln!("Unknown option: {opt:?}!");
                return ExitCode::FAILURE;
            }
        }
    }

    #[allow(clippy::disallowed_methods)]
    match opts.positionals().next() {
        None | Some("-") => {
            if bino {
                // Binary output for standard input
                std::io::stdout()
                    .write_all(&syd::hash::hash(std::io::stdin().lock(), func).expect("hash"))
                    .expect("write to stdout");
            } else {
                // Hexadecimal output for standard input
                println!(
                    "{} -",
                    const_hex::encode(
                        syd::hash::hash(std::io::stdin().lock(), func).expect("hash")
                    )
                );
            }
        }
        Some(path) => {
            let file = BufReader::new(File::open(path).expect("open file"));
            if bino {
                // Binary output for file input
                std::io::stdout()
                    .write_all(&syd::hash::hash(file, func).expect("hash"))
                    .expect("write to stdout");
            } else {
                // Hexadecimal output for file input
                println!(
                    "{} {path}",
                    const_hex::encode(syd::hash::hash(file, func).expect("hash"))
                );
            }
        }
    }

    ExitCode::SUCCESS
}

fn help() {
    println!("Usage: syd-sha [-bhm1235] <file|->");
    println!("Given a file, print the SHA3-512 checksum of the file.");
    println!("Given no positional arguments, calculate the SHA3-512 checksum of standard input.");
    println!("Use -m to calculate MD5 instead of SHA3-512 (\x1b[91minsecure\x1b[0m, \x1b[96mPortage\x1b[0m/\x1b[95mPaludis\x1b[0m vdb compat).");
    println!("Use -1 to calculate SHA1 instead of SHA3-512 (\x1b[91minsecure\x1b[0m).");
    println!("Use -2 to calculate SHA3-256 instead of SHA3-512.");
    println!("Use -3 to calculate SHA3-384 instead of SHA3-512.");
    println!("Use -b to print binary output rather than hex-encoded string.");
}
